/* Copyright (c) 2009 InMon Corp. ALL RIGHTS RESERVED */
/* License: http://www.inmon.com/products/virtual-probe/license.php */

#ifndef VSPROBED_H
#define VSPROBED_H 1

#if defined(__cplusplus)
extern "C" {
#endif

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <errno.h>
#include <netdb.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <syslog.h>
#include <signal.h>
#include <fcntl.h>
#include <assert.h>
#include <ctype.h>
#include <pcap.h>

#include "sflow_api.h"

#define YES 1
#define NO 0

#define VSP_VERSION "1.0.1"
#define VSP_DAEMON_NAME "vsprobed"
#define VSP_DEFAULT_PIDFILE "/var/run/vsprobed.pid"
#define VSP_DEFAULT_CONFIGFILE "/etc/vsprobed.conf"
#define VSP_DEFAULT_PROMISCUOUS 1
#define VSP_DEFAULT_READ_BATCH 100

  // forward declarations
  struct _VSPSwitch;
  struct _VSPSFlow;
  struct _VSP;

  typedef struct _VSPPCAP {
    char *device;
    pcap_t *pcap;
    int fd;
    u_int32_t snaplen;
    u_int32_t batch;
    u_int32_t promiscuous;
    char errbuf[PCAP_ERRBUF_SIZE];
  } VSPPCAP;

  typedef struct _VSPSFlowSettings {
    u_int32_t samplingRate;
    u_int32_t pollingInterval;
  } VSPSFlowSettings;

  typedef struct _VSPCollector {
    struct _VSPCollector *nxt;
    SFLAddress ipAddr;
    u_int32_t udpPort;
    struct sockaddr_in6 sendSocket;
  } VSPCollector;

  typedef struct _VSPInterface {
    struct _VSPInterface *nxt;
    int uplink;
    struct _VSPSwitch *mySwitch;
    u_int64_t mac;
    u_int32_t ifIndex;
    u_int32_t ifType;
    u_int64_t ifSpeed;
    // SFLSampler *sampler;
    SFLPoller *poller;
    // keep the block of interface counters here too
    SFLCounters_sample_element genElem;
  } VSPInterface;

  typedef struct _VSPSwitch {
    struct _VSPSwitch *nxt;
    struct _VSPSFlow *mySFlow;
    VSPPCAP *pktReader;
    u_int32_t subAgentId; 
    SFLAgent *agent;
    VSPInterface *interfaces;
    u_int32_t numInterfaces;
    VSPInterface *uplink;
    SFLSampler *sampler;
    // macToInterface lookup table here
  } VSPSwitch;

  typedef struct _VSPSFlow {
    struct _VSP *myVSP;
    VSPSwitch *switches;
    u_int32_t numSwitches;
    VSPCollector *collectors;
    u_int32_t numCollectors;
    VSPSFlowSettings *sFlowSettings;
    u_int32_t firstIfIndex;
    u_int32_t firstSubAgentId;
    SFLAddress agentIP;
    struct sockaddr_in6 agentSocket;
    u_int64_t macBroadcast;
  } VSPSFlow; 

  typedef struct _VSP {
    int debug;
    VSPSFlow *sFlow;
    char *configFile;
    char *pidFile;
  } VSP;

  // config parser
  int VSPReadConfigFile(VSP *sp);


#if defined(__cplusplus)
} /* extern "C" */
#endif

#endif /* VSPROBED_H */
